==> common-search-1.0-20131018.095843-5-sources.jar: cn/pconline/search/common/tools/sensitive/AnalyzeSensitiveFilter.java <==
package cn.pconline.search.common.tools.sensitive;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.locks.ReentrantLock;

import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;

import cn.pconline.search.common.Configuration;
import cn.pconline.search.common.util.DaliyFixTimeRunThread;

/**
 * 基于对输入进行分析匹配的敏感词过滤器
 * 
 * @author zengjie
 * @since 2013-10-9
 * @see
 */
public class AnalyzeSensitiveFilter extends DaliyFixTimeRunThread implements
        SensitiveFilter
{

    private ReentrantLock loadLock = new ReentrantLock();

    private String wordDb;

    protected volatile WordMapping mapping = null;

    private String domain;

    public AnalyzeSensitiveFilter(Configuration config)
    {
        super(config.getConfig("sensitiveWordReloadTime", ""),
                "SensitiveWord-Load-Thread");
        wordDb = config.getConfig("sensitiveWordDb");
        domain = config.getConfig("sensitiveFilterDomain", "pconline");
        doLoad();
    }

    @Override
    protected void doRun()
    {
        doLoad();
    }

    @Override
    public SensitiveMatcher matchSensitive(String input)
    {
        return hasSensitiveInMapping(input, mapping);
    }

    private SensitiveMatcher hasSensitiveInMapping(String input,
            WordMapping mapping)
    {
        if (StringUtils.isBlank(input))
        {
            return SensitiveMatcher.EMPTY_MATCHER;
        }
        input = input.trim();
        if (input.length() < mapping.getMinWordLen())
        {
            return SensitiveMatcher.EMPTY_MATCHER;
        }
        if (!mapping.hasMapping())
        {
            return SensitiveMatcher.EMPTY_MATCHER;
        }
        return new SensitiveMatcherImpl(mapping, input);
    }

    @Override
    public void reload(SensitiveWordSource source)
    {
        loadLock.lock();
        try
        {
            WordMapping mapping = new WordMapping();
            SensitiveWord word = null;
            while ((word = source.nextWord()) != null)
            {
                mapping.addWord(word);
                this.mapping = mapping;
            }
        }
        finally
        {
            loadLock.unlock();
        }
    }

    private void doLoad()
    {

        OracleSensitiveWordSource source = null;
        try
        {
            loadLock.lock();
            source = new OracleSensitiveWordSource(wordDb, domain);
            this.reload(source);
        }
        finally
        {
            loadLock.unlock();
            if (source != null) source.close();

        }
    }

    static class WordMapping
    {

        private int minWordLen = Integer.MAX_VALUE;

        private int maxWordLen = 0;

        private Map<String, SensitiveWord> wordMap = new HashMap<String, SensitiveWord>();

        public void addWord(SensitiveWord word)
        {
            wordMap.put(word.getWord(), word);
            minWordLen = Math.min(word.getWord().length(), minWordLen);
            maxWordLen = Math.max(word.getWord().length(), maxWordLen);
        }

        public SensitiveWord hasWord(String word)
        {
            return wordMap.get(word);
        }

        public int getMaxWordLen()
        {
            return maxWordLen;
        }

        public boolean hasMapping()
        {
            return MapUtils.isNotEmpty(wordMap);
        }

        public int getMinWordLen()
        {
            return minWordLen;
        }

    }
}

==> common-search-1.0-20131112.061904-8-sources.jar: cn/pconline/search/common/tools/sensitive/AnalyzeSensitiveFilter.java <==
package cn.pconline.search.common.tools.sensitive;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.locks.ReentrantLock;

import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;

import cn.pconline.search.common.Configuration;
import cn.pconline.search.common.util.DaliyFixTimeRunThread;

/**
 * 基于对输入进行分析匹配的敏感词过滤器
 * 
 * @author zengjie
 * @since 2013-10-9
 * @see
 */
public class AnalyzeSensitiveFilter extends DaliyFixTimeRunThread implements
        SensitiveFilter
{

    private ReentrantLock loadLock = new ReentrantLock();

    private String wordDb;

    protected volatile WordMapping mapping = null;

    private String domain;

    public AnalyzeSensitiveFilter(Configuration config)
    {
        super(config.getConfig("sensitiveWordReloadTime", ""),
                "SensitiveWord-Load-Thread");
        wordDb = config.getConfigNotEmpty("sensitiveWordDb");
        domain = config.getConfig("sensitiveFilterDomain", "pconline");
        doLoad();
    }

    @Override
    protected void doRun()
    {
        doLoad();
    }

    @Override
    public SensitiveMatcher matchSensitive(String input)
    {
        return hasSensitiveInMapping(input, mapping);
    }

    private SensitiveMatcher hasSensitiveInMapping(String input,
            WordMapping mapping)
    {
        if (StringUtils.isBlank(input))
        {
            return SensitiveMatcher.EMPTY_MATCHER;
        }
        input = input.trim();
        if (input.length() < mapping.getMinWordLen())
        {
            return SensitiveMatcher.EMPTY_MATCHER;
        }
        if (!mapping.hasMapping())
        {
            return SensitiveMatcher.EMPTY_MATCHER;
        }
        return new SensitiveMatcherImpl(mapping, input);
    }

    @Override
    public void reload(SensitiveWordSource source)
    {
        loadLock.lock();
        try
        {
            WordMapping mapping = new WordMapping();
            SensitiveWord word = null;
            while ((word = source.nextWord()) != null)
            {
                mapping.addWord(word);
                this.mapping = mapping;
            }
        }
        finally
        {
            loadLock.unlock();
        }
    }

    private void doLoad()
    {

        OracleSensitiveWordSource source = null;
        try
        {
            loadLock.lock();
            source = new OracleSensitiveWordSource(wordDb, domain);
            this.reload(source);
        }
        finally
        {
            loadLock.unlock();
            if (source != null) source.close();

        }
    }

    static class WordMapping
    {

        private int minWordLen = Integer.MAX_VALUE;

        private int maxWordLen = 0;

        private Map<String, SensitiveWord> wordMap = new HashMap<String, SensitiveWord>();

        public void addWord(SensitiveWord word)
        {
            wordMap.put(word.getWord(), word);
            minWordLen = Math.min(word.getWord().length(), minWordLen);
            maxWordLen = Math.max(word.getWord().length(), maxWordLen);
        }

        public SensitiveWord hasWord(String word)
        {
            return wordMap.get(word);
        }

        public int getMaxWordLen()
        {
            return maxWordLen;
        }

        public boolean hasMapping()
        {
            return MapUtils.isNotEmpty(wordMap);
        }

        public int getMinWordLen()
        {
            return minWordLen;
        }

    }
}
